@@ -7,7 +7,7 @@ import matplotlib.pyplot as plt
 from pprint import pprint
 
 from .cli import splash, list_menu, app_menu, input_prompt
-from .io import load_config
+from .io import load_config, get_logger
 
 from ..core.api import StoppingProblem
 from ..core.schema import Adapter, ExperimentSchema
@@ -32,10 +32,36 @@ def simulate(experiment_config, backend_config):
     return 0
 
 
-def optimise(config, **kwargs):
+def optimise(
+    experiment_config, backend_config, steps=100, initial_lr=1, decay_constant=1.5
+):
 
-    print("Stand-in for optimisation")
+    backend = Backend(**backend_config)
+    logger = get_logger("optim.log")
 
+    stoppingProblem = StoppingProblem.from_config(experiment_config)
+
+    param_history, cost_history = stoppingProblem.optimise(
+        backend=backend,
+        steps=steps,
+        initial_lr=initial_lr,
+        decay_constant=decay_constant,
+    )
+
+    experiment_config = dict(stoppingProblem)  # Get the optimised results
+    result = stoppingProblem.run()
+    adapter = Adapter()
+
+    experiment_config["results"] = result
+    experiment_config["results"]["timestamp"] = datetime.now().isoformat()
+    experiment_config["solver"] = {}
+    experiment_config["solver"]["parameter_history"] = param_history
+    experiment_config["solver"]["cost_history"] = cost_history
+    experiment_config["solver"]["initial_lr"] = initial_lr
+    experiment_config["solver"]["decay_constant"] = decay_constant
+
+    experiment_dict = Adapter.jsonify(experiment_config)
+    backend.save(experiment_dict)
     return 0
 
 
@@ -132,7 +158,7 @@ def get_app_list():
 
 def launch(args):
 
-    splash("Quickest")
+    splash("gotta go fast")
 
     if args.application:
         application_str = args.application
@@ -155,11 +181,53 @@ def launch(args):
 
     else:
         print("No config supplied - creating one now")
-        choices = list_menu(StoppingProblem.get_opts())
+        # NUM DIMENSIONS
+        # NUM PRE AND POST-CHANGE STATES
+
+        num_states = int(input_prompt(msg="Enter belief space dimensions", default="3"))
+
+        num_pre_change_states = num_states - int(
+            input_prompt(msg="Enter the number of post-change states", default="1")
+        )
+
+        class_opts = list_menu(StoppingProblem.get_opts())
+
+        choices = {
+            "num_states": num_states,
+            "num_pre_change_states": num_pre_change_states,
+            **class_opts,
+        }
         config_dict = StoppingProblem.get_config(choices)
 
-    if application_str == 'getconf':
+    if application_str == "getconf":
         return get_config(config_dict)
+    elif application_str == "optimise":
+        if not args.steps:
+            steps = 100
+        else:
+            steps = args.steps
+
+        if not args.lr:
+            initial_lr = 1
+        else:
+            initial_lr = args.lr
+
+        if not args.decay:
+            decay_constant = 1.5
+        else:
+            decay_constant = args.decay
+        print(
+            "Optimising for {} steps with initial learning rate {} and decay constant {}".format(
+                steps, initial_lr, decay_constant
+            )
+        )
+        return optimise(
+            config_dict,
+            backend_config,
+            steps=steps,
+            initial_lr=initial_lr,
+            decay_constant=decay_constant,
+        )
     else:
         return application_types[application_str](config_dict, backend_config)
 
